// This is the main DLL file.

#include "stdafx.h"

#include "WRRCU100APIWrapper.h"

using namespace WRRCU100APIWrapperSpace;
using namespace System;
using namespace System::Runtime::InteropServices;


//
// Notification
//
void __stdcall WRRCU100APIWrapperNotification::RegisterCallbacks(POSITION_CHANGED_CALLBACK thePositionCallback, STATE_CHANGED_CALLBACK theStateCallback) // Callbacks from unmanaged to managed C++
{
	positionCallback = thePositionCallback;
	stateCallback = theStateCallback;
}

void __stdcall WRRCU100APIWrapperNotification::PositionChanged(IWRRCU100* theDevice, INT32 theAzimuth, INT32 theElevation)
{
	Console::WriteLine("Notification::PositionChanged: azimuth = {0}, elevation = {1}", theAzimuth, theElevation);

	positionCallback(theAzimuth, theElevation); // Callback to managed C++

	if (theAzimuth == 0 && theElevation == 0)
	{
		Console::WriteLine("Notification::PositionChanged -> PostQuitMessage");
	}
}

void __stdcall WRRCU100APIWrapperNotification::StateChanged(IWRRCU100* theDevice, UINT32 theState)
{
	Console::WriteLine("Notification::StateChanged: {0}", theState);

	stateCallback(theState); // Callback to managed C++
}


//
// ApiHandler
//
HMODULE ourModuleHandle = NULL;
CREATE_INSTANCE ourCreateInstance;
IWRRCU100* ourDevice = NULL;
WRRCU100APIWrapperNotification ourNotification;

WRRCU100APIWrapper::WRRCU100APIWrapper()
{
	instance = this; // Used in callback from managed C++ to C#

	IntPtr aPointer = Marshal::StringToHGlobalUni(APILIBRARY);
	ourModuleHandle = LoadLibrary((LPCWSTR)aPointer.ToPointer());

	if (ourModuleHandle != NULL)
	{
		Console::WriteLine("Library loaded...");

		ourCreateInstance = (CREATE_INSTANCE)GetProcAddress(ourModuleHandle, "CreateInstance");

		status = STATUS_APILIBRARY_LOADED;
	}
	else
	{
		Console::WriteLine("Failed to load library");

		status = STATUS_APILIBRARY_NOT_FOUND;
	}
}

WRRCU100APIWrapper::~WRRCU100APIWrapper()
{
	FreeLibrary(ourModuleHandle);
}

void WRRCU100APIWrapper::NotificationPositionChangedCallback(INT32 theAzimuth, INT32 theElevation)
{
	Console::WriteLine("ApiHandler::NotificationPositionChangedCallback: theAzimuth = {0}, theElevation = {1}", theAzimuth, theElevation);

	instance->positionCallback(theAzimuth, theElevation);
}

void WRRCU100APIWrapper::NotificationStateChangedCallback(UINT32 theState)
{
	instance->stateCallback(theState);
}

DWORD WRRCU100APIWrapper::GetStatus()
{
	return status;
}

BOOL WRRCU100APIWrapper::IsOpened()
{
	if (ourDevice == NULL)
	{
		return FALSE;
	}
	else
	{
		return ourDevice->IsOpen();
	}
}

long WRRCU100APIWrapper::GetId()
{
	return deviceInfo.id;
}

String^ WRRCU100APIWrapper::GetSerialNumber()
{
	return deviceInfo.serialNumber;
}

BOOL WRRCU100APIWrapper::Enumerate()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	IWRRCU100DeviceList* aDeviceList;
	WRRCU100_DEVICE_INFO aDeviceInfo;

	BOOL aResult = FALSE;

	if (ourCreateInstance(WRRCU100_CLASS_ID_DEVICE_LIST, (void**)&aDeviceList))
	{
        DWORD anIndex = 0;

        aDeviceList->BeginEnumeration();

        if (aDeviceList->Next(&aDeviceInfo))
        {
			deviceInfo.id = aDeviceInfo.Id;
			deviceInfo.serialNumber = gcnew String(aDeviceInfo.SerialNumber);
			Console::WriteLine("#{0}, serial number: {1}", deviceInfo.id, deviceInfo.serialNumber);

			aResult = TRUE;
        }
		else
		{
			Console::WriteLine("No available WR-RCU-100 device found");
		}

        aDeviceList->EndEnumeration();

        aDeviceList->Release();
    }
    else
    {
        Console::WriteLine("Failed to create device list object\n");
    }

	return aResult;
}

BOOL WRRCU100APIWrapper::Open(PositionChangedCallback^ thePositionCallback, StateChangedCallback^ theStateCallback)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (!ourCreateInstance(WRRCU100_CLASS_ID_DEVICE, (void**)&ourDevice))
	{
		Console::WriteLine("Failed to create device");

		return FALSE;
	}

	ourDevice->Open(deviceInfo.id);

	positionCallback = thePositionCallback; // Callback to C#
	stateCallback = theStateCallback; // Callback to C#
	ourDevice->RegisterNotification(&ourNotification);

	// Position-changed-callback from unmanaged to managed C++
	NotificationPositionChangedDelegate^ aPositionDelegate = gcnew NotificationPositionChangedDelegate(NotificationPositionChangedCallback);
	GCHandle aHandle = GCHandle::Alloc(aPositionDelegate);
	IntPtr aPointer = Marshal::GetFunctionPointerForDelegate(aPositionDelegate);
	POSITION_CHANGED_CALLBACK aPositionCallback = static_cast<POSITION_CHANGED_CALLBACK>(aPointer.ToPointer());

	// State-changed-callback from unmanaged to managed C++
	NotificationStateChangedDelegate^ aStateDelegate = gcnew NotificationStateChangedDelegate(NotificationStateChangedCallback);
	aHandle = GCHandle::Alloc(aStateDelegate);
	aPointer = Marshal::GetFunctionPointerForDelegate(aStateDelegate);
	STATE_CHANGED_CALLBACK aStateCallback = static_cast<STATE_CHANGED_CALLBACK>(aPointer.ToPointer());

	// Force garbage collection cycle to prove that the delegate does not get disposed
	GC::Collect();
	ourNotification.RegisterCallbacks(aPositionCallback, aStateCallback);

	Console::WriteLine("Device opened");

	return TRUE;
}

BOOL WRRCU100APIWrapper::Close()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (ourDevice != NULL)
	{
		ourDevice->Close();
		ourDevice->Release();

		ourDevice = NULL;

		Console::WriteLine("Device closed");
	}

	return TRUE;
}

String^ WRRCU100APIWrapper::GetFirmwareVersion()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return "";
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return "";
	}

	char aBuffer[1000];
	ourDevice->GetFirmwareVersion(aBuffer, sizeof(aBuffer));

	Console::WriteLine("Device firmware version: {0}", gcnew String(aBuffer));

	return gcnew String(aBuffer);
}

UINT32 WRRCU100APIWrapper::GetState()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	UINT32 aState;
	ourDevice->GetState(&aState);

	Console::WriteLine("Device state: {0}", aState);

	return aState;
}

BOOL WRRCU100APIWrapper::StartCalibration()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	Console::WriteLine("Starting calibration...");

	return ourDevice->StartCalibration(FALSE);
}

void WRRCU100APIWrapper::StopCalibration()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return;
	}

	ourDevice->StopCalibration();

	Console::WriteLine("Calibration stopped");
}

BOOL WRRCU100APIWrapper::SetPosition(INT32 theAzimuth, INT32 theElevation)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	Console::WriteLine("Set position: azimuth = {0}, elevation = {1}", theAzimuth, theElevation);

	return ourDevice->SetPosition(theAzimuth, theElevation);
}

WRRCU100APIWrapperPosition^ WRRCU100APIWrapper::GetPosition()
{
	WRRCU100APIWrapperPosition^ aPosition;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return aPosition;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return aPosition;
	}

	INT32 anAzimuth, anElevation;
	if (ourDevice->GetPosition(&anAzimuth, &anElevation))
	{
		aPosition = gcnew WRRCU100APIWrapperPosition();
		aPosition->azimuth = anAzimuth;
		aPosition->elevation = anElevation;
	}

	return aPosition;
}

WRRCU100APIWrapperAzimuthRange^ WRRCU100APIWrapper::GetAzimuthRange()
{
	WRRCU100APIWrapperAzimuthRange^ aRange;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return aRange;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return aRange;
	}

	INT32 aMinAzimuth, aMaxAzimuth;
	if (ourDevice->GetAzimuthRange(&aMinAzimuth, &aMaxAzimuth))
	{
		aRange = gcnew WRRCU100APIWrapperAzimuthRange();
		aRange->min = aMinAzimuth;
		aRange->max = aMaxAzimuth;
	}

	return aRange;
}

WRRCU100APIWrapperElevationRange^ WRRCU100APIWrapper::GetElevationRange()
{
	WRRCU100APIWrapperElevationRange^ aRange;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return aRange;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return aRange;
	}

	INT32 aMinElevation, aMaxElevation;
	if (ourDevice->GetElevationRange(&aMinElevation, &aMaxElevation))
	{
		aRange = gcnew WRRCU100APIWrapperElevationRange();
		aRange->min = aMinElevation;
		aRange->max = aMaxElevation;
	}

	return aRange;
}

BOOL WRRCU100APIWrapper::SetOffsets(INT32 theAzimuth, INT32 theElevation)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	Console::WriteLine("Set offsets: azimuth = {0}, elevation = {1}", theAzimuth, theElevation);

	return ourDevice->SetOffsets(theAzimuth, theElevation);
}

WRRCU100APIWrapperOffsets^ WRRCU100APIWrapper::GetOffsets()
{
	WRRCU100APIWrapperOffsets^ anOffsets;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return anOffsets;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return anOffsets;
	}

	INT32 anAzimuth, anElevation;
	if (ourDevice->GetOffsets(&anAzimuth, &anElevation))
	{
		anOffsets = gcnew WRRCU100APIWrapperOffsets();
		anOffsets->azimuth = anAzimuth;
		anOffsets->elevation = anElevation;
	}

	return anOffsets;
}

WRRCU100APIWrapperAzimuthOffsetRange^ WRRCU100APIWrapper::GetAzimuthOffsetRange()
{
	WRRCU100APIWrapperAzimuthOffsetRange^ aRange;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return aRange;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return aRange;
	}

	INT32 aMinOffset, aMaxOffset;
	if (ourDevice->GetAzimuthOffsetRange(&aMinOffset, &aMaxOffset))
	{
		aRange = gcnew WRRCU100APIWrapperAzimuthOffsetRange();
		aRange->min = aMinOffset;
		aRange->max = aMaxOffset;
	}

	return aRange;
}

WRRCU100APIWrapperElevationOffsetRange^ WRRCU100APIWrapper::GetElevationOffsetRange()
{
	WRRCU100APIWrapperElevationOffsetRange^ aRange;

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Library not loaded");

		return aRange;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return aRange;
	}

	INT32 aMinOffset, aMaxOffset;
	if (ourDevice->GetElevationOffsetRange(&aMinOffset, &aMaxOffset))
	{
		aRange = gcnew WRRCU100APIWrapperElevationOffsetRange();
		aRange->min = aMinOffset;
		aRange->max = aMaxOffset;
	}

	return aRange;
}