﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using WRRCU100APIWrapperSpace;

namespace WRRCU100Example
{
    public partial class MainForm : Form
    {
        #region Members

        Wrapper wrapper = new Wrapper();
        bool isCalibrationStarted;

        const uint WRRCU100_STATE_READY = 0x00000000;
        const uint WRRCU100_STATE_DISCONNECTED = 0x00000001;
        const uint WRRCU100_STATE_HIGH_TEMPERATURE = 0x00000002;
        const uint WRRCU100_STATE_LOW_VOLTAGE = 0x00000004;
        const uint WRRCU100_STATE_OVERLOAD_AZ = 0x00000008;
        const uint WRRCU100_STATE_OVERLOAD_EL = 0x00000010;
        const uint WRRCU100_STATE_AZ_IS_NOT_MOVING = 0x00000020;
        const uint WRRCU100_STATE_EL_IS_NOT_MOVING = 0x00000040;
        const uint WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN = 0x00000080;
        const uint WRRCU100_STATE_CALIBRATION_REQUIRED = 0x00000100;
        const uint WRRCU100_STATE_CALIBRATION_IN_PROGRESS = 0x00000200;

        #endregion

        #region Constructors

        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Gui Event Handlers

        private void OnMainFormLoad(object theSender, EventArgs theEventArg)
        {
            wrapper.devicePositionChanged += new Wrapper.DevicePositionChangedHandler(OnWrapperDevicePositionChanged);
            wrapper.deviceStateChanged += new Wrapper.DeviceStateChangedHandler(OnWrapperDeviceStateChanged);

            if (wrapper.Init())
            {
                WRRCU100APIWrapperDeviceInfo aDeviceInfo = wrapper.deviceInfo;

                toolStripStatusLabel.Text = "Device #" + aDeviceInfo.id + " found";
                textBoxSerialNumber.Text = wrapper.deviceInfo.serialNumber;
                textBoxFirmwareVersion.Text = wrapper.deviceFirmwareVersion;

                panelRotator.Enabled = true;

                HandleDeviceState(wrapper.deviceState);

                HandleControls();
            }
            else
            {
                toolStripStatusLabel.ForeColor = Color.Red;
                if (wrapper.apiStatus == Wrapper.APISTATUS_WRAPPERLIBRARY_NOTFOUND)
                {
                    toolStripStatusLabel.Text = "Unable to find " + Wrapper.APIWRAPPERLIBRARY + " library!";
                }
                else
                {
                    toolStripStatusLabel.Text = "Unable to find device!";
                }
            }
        }

        private void OnMainFormFormClosing(object theSender, FormClosingEventArgs theEventArg)
        {
            if (wrapper.IsDeviceOpened() != 0)
            {
                wrapper.CloseDevice();
            }
        }

        private void OnButtonCalibrationStartClick(object theSender, EventArgs theEventArgs)
        {
            if (wrapper.StartDeviceCalibration())
            {
                isCalibrationStarted = true;

                StartCalibrationProgressBar();

                HandleControls();

                toolStripStatusLabel.Text = "Calibration started...";
            }
        }

        private void OnButtonCalibrationStopClick(object theSender, EventArgs theEventArgs)
        {
            wrapper.StopDeviceCalibration();

            isCalibrationStarted = false;

            StopCalibrationProgressBar(false);

            buttonCalibrationStart.Enabled = true;
            buttonCalibrationStop.Enabled = false;

            toolStripStatusLabel.Text = "Calibration stopped";
        }

        private void NumericUpDownAzimuthTargetValueChanged(object theSender, EventArgs theEventArgs)
        {
            wrapper.SetDevicePosition((Int32)numericUpDownAzimuthTarget.Value, (Int32)numericUpDownElevationTarget.Value);
        }

        private void NumericUpDownElevationTargetValueChanged(object theSender, EventArgs theEventArgs)
        {
            wrapper.SetDevicePosition((Int32)numericUpDownAzimuthTarget.Value, (Int32)numericUpDownElevationTarget.Value);
        }

        #endregion

        #region Wrapper Event Handlers

        private void OnWrapperDevicePositionChanged(object theSender, EventArgs theEventArgs)
        {
            DeviceEventArgs anEventArgs = (DeviceEventArgs)theEventArgs;

            textBoxAzimuthRotator.Text = anEventArgs.azimuth.ToString();
            textBoxElevationRotator.Text = anEventArgs.elevation.ToString();
        }

        private void OnWrapperDeviceStateChanged(object theSender, EventArgs theEventArgs)
        {
            DeviceEventArgs anEventArgs = (DeviceEventArgs)theEventArgs;

            // If finish
            if (isCalibrationStarted && (anEventArgs.state & WRRCU100_STATE_CALIBRATION_IN_PROGRESS) == 0)
            {
                isCalibrationStarted = false;

                HandleControls();

                if ((anEventArgs.state & WRRCU100_STATE_CALIBRATION_REQUIRED) == WRRCU100_STATE_CALIBRATION_REQUIRED)
                {
                    StopCalibrationProgressBar(false);

                    toolStripStatusLabel.Text = "Calibration failed!";
                }
                else
                {
                    StopCalibrationProgressBar(true);

                    toolStripStatusLabel.Text = "Calibration successful";
                }
            }

            HandleDeviceState(anEventArgs.state);
        }

        #endregion

        #region Private Methods

        private void StartCalibrationProgressBar()
        {
            progressBarCalibration.Style = ProgressBarStyle.Marquee;
        }

        private void StopCalibrationProgressBar(bool theSuccessful)
        {
            progressBarCalibration.Style = ProgressBarStyle.Continuous;
            progressBarCalibration.Value = theSuccessful ? progressBarCalibration.Maximum : progressBarCalibration.Minimum;
        }

        private void HandleControls()
        {
            if (isCalibrationStarted)
            {
                buttonCalibrationStart.Enabled = false;
                buttonCalibrationStop.Enabled = true;
            }
            else
            {
                buttonCalibrationStart.Enabled = true;
                buttonCalibrationStop.Enabled = false;
            }

            UInt32 aState = wrapper.GetDeviceState();
            if (!isCalibrationStarted && (aState & WRRCU100_STATE_CALIBRATION_REQUIRED) != WRRCU100_STATE_CALIBRATION_REQUIRED)
            {
                groupBoxAzimuth.Enabled = true;
                groupBoxElevation.Enabled = true;
            }
            else
            {
                groupBoxAzimuth.Enabled = false;
                groupBoxElevation.Enabled = false;
            }
        }

        private void HandleDeviceState(uint theState)
        {
            textBoxAzimuthRange.Text = "";
            textBoxAzimuthRotator.Text = "";

            textBoxElevationRange.Text = "";
            textBoxElevationRotator.Text = "";

            numericUpDownAzimuthTarget.Minimum = numericUpDownAzimuthTarget.Maximum = numericUpDownElevationTarget.Minimum = numericUpDownElevationTarget.Maximum = 0;

            listBoxState.Items.Clear();
            ListBox.ObjectCollection anItems = listBoxState.Items;

            // If device is calibrated and ready to use
            if (theState == WRRCU100_STATE_READY)
            {
                if (DisplayPositionInfos())
                {
                    groupBoxAzimuth.Enabled = true;
                    groupBoxElevation.Enabled = true;

                    anItems.Add("Device ready");
                }
                else
                {
                    groupBoxAzimuth.Enabled = false;
                    groupBoxElevation.Enabled = false;

                    anItems.Add("Unknown error");
                }
            }
            else
            {
                if ((theState & WRRCU100_STATE_DISCONNECTED) == WRRCU100_STATE_DISCONNECTED)
                {
                    anItems.Add("Device disconnected");
                }
                if ((theState & WRRCU100_STATE_HIGH_TEMPERATURE) == WRRCU100_STATE_HIGH_TEMPERATURE)
                {
                    anItems.Add("High internal temperature");
                }
                if ((theState & WRRCU100_STATE_LOW_VOLTAGE) == WRRCU100_STATE_LOW_VOLTAGE)
                {
                    anItems.Add("Low voltage");
                }
                if ((theState & WRRCU100_STATE_OVERLOAD_AZ) == WRRCU100_STATE_OVERLOAD_AZ)
                {
                    anItems.Add("Azimuth motor overloaded");
                }
                if ((theState & WRRCU100_STATE_OVERLOAD_EL) == WRRCU100_STATE_OVERLOAD_EL)
                {
                    anItems.Add("Elevation motor overloaded");
                }
                if ((theState & WRRCU100_STATE_AZ_IS_NOT_MOVING) == WRRCU100_STATE_AZ_IS_NOT_MOVING)
                {
                    anItems.Add("Azimuth not moving");
                }
                if ((theState & WRRCU100_STATE_EL_IS_NOT_MOVING) == WRRCU100_STATE_EL_IS_NOT_MOVING)
                {
                    anItems.Add("Elevation not moving");
                }
                if ((theState & WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN) == WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN)
                {
                    anItems.Add("Mechanical ranges unknown");
                }
                if ((theState & WRRCU100_STATE_CALIBRATION_REQUIRED) == WRRCU100_STATE_CALIBRATION_REQUIRED)
                {
                    anItems.Add("Calibration required");
                }
                if ((theState & WRRCU100_STATE_CALIBRATION_IN_PROGRESS) == WRRCU100_STATE_CALIBRATION_IN_PROGRESS)
                {
                    anItems.Add("Calibration in progress");
                }
            }
        }

        private bool DisplayPositionInfos()
        {
            DeviceOffsets aPosition = new DeviceOffsets();
            DeviceRanges aRanges = new DeviceRanges();
            DeviceOffsets anOffsets = new DeviceOffsets();
            DeviceOffsetRanges anOffsetRanges = new DeviceOffsetRanges();

            bool aPositionResult = wrapper.GetDeviceRanges(ref aPosition),
                aRangesResult = wrapper.GetDeviceRanges(ref aRanges),
                anOffsetsResult = wrapper.GetDeviceOffsets(ref anOffsets),
                anOffsetRangesResult = wrapper.GetDeviceOffsetRanges(ref anOffsetRanges);

            if (aPositionResult && aRangesResult && anOffsetsResult && anOffsetRangesResult)
            {
                // Ranges
                Int32 anAzimuthRangeMin = aRanges.minAzimuth, anAzimuthRangeMax = aRanges.maxAzimuth, anElevationRangeMin = aRanges.minElevation, anElevationRangeMax = aRanges.maxElevation;
                textBoxAzimuthRange.Text = FormatRangeString(anAzimuthRangeMin, anAzimuthRangeMax);
                numericUpDownAzimuthTarget.Minimum = anAzimuthRangeMin;
                numericUpDownAzimuthTarget.Maximum = anAzimuthRangeMax;
                textBoxElevationRange.Text = FormatRangeString(anElevationRangeMin, anElevationRangeMax);
                numericUpDownElevationTarget.Minimum = anElevationRangeMin;
                numericUpDownElevationTarget.Maximum = anElevationRangeMax;

                // Position
                Int32 anAzimuth = aPosition.azimuth, anElevation = aPosition.elevation;
                textBoxAzimuthRotator.Text = anAzimuth.ToString();
                textBoxElevationRotator.Text = anElevation.ToString();
                numericUpDownAzimuthTarget.Value = anAzimuth;
                numericUpDownElevationTarget.Value = anElevation;

                aPosition = null;
                aRanges = null;
                anOffsets = null;
                anOffsetRanges = null;

                return true;
            }
            else
            {
                return false;
            }
        }

        private String FormatRangeString(Int32 theMin, Int32 theMax)
        {
            return theMin + " to " + theMax;
        }

        #endregion
    }
}
